(* module Syntax: syntax trees and associated support functions *)

open Support.Pervasive
open Support.Error

(* Data type definitions *)
type ty =
    TyBool
  | TyNat

type term =
    TmTrue of info
  | TmFalse of info
  | TmIf of info * term * term * term
  | TmZero of info
  | TmSucc of info * term
  | TmPred of info * term
  | TmIsZero of info * term
  | TmVar of info * string
  | TmFunApp of info * string * term list

type binding =
    VarBind of ty

type progdef = 
    FnDef of info * (string * ty) list * ty * (term option)

type command =
    Eval of info * term
  | UnsafeEval of info * term
  | Bind of info * string * binding
  | Prog of info * (string * progdef) list

(* Contexts *)
type context
val emptycontext : context
val isbound : context -> string -> bool
val addbinding : info -> context -> string * binding -> context
val getTypeFromContext : info -> context -> string -> ty

(* Programs *)
type prog
val emptyprog : prog
val makeprog		  : (string * progdef) list -> prog
val getFunBodyFromProgram : info -> prog -> string -> string list * term option
val getFunTypeFromProgram : info -> prog -> string -> ty list * ty
val hasbodyFun : prog -> string -> bool

(* Substitution *)
val simultSubstArgsVals : info -> string list -> term list -> term -> term

(* Printing *)
val printtm: term -> unit
val printty : ty -> unit
val prbinding : binding -> unit
val prprog : prog -> unit

(* Misc *)
val tmInfo: term -> info

